@@ -41,82 +41,88 @@ typedef struct lnf_brec1_s {
 #define LNF_BASIC_RECORD1	0xB1
 
 
-#define LNF_MASK_TYPE  		0x000000FF
+#define LNF_MASK_TYPE  		0x0000FF
+#define LNF_MASK_FIELD 		0xFFFF00 
+
 #define LNF_GET_TYPE(x) 	(x & LNF_MASK_TYPE)
+//#define LNF_GET_FIELD(x) 	((x & LNF_MASK_FIELD) >> 8)
+#define LNF_GET_FIELD(x) 	x
 
 /* top two bytes of field identifies data type LNF_UINT8, ... */
 
 #define LNF_FLD_ZERO			0x00
-#define LNF_FLD_FIRST			( 0x010000 | LNF_UINT64 )
-#define LNF_FLD_LAST			( 0x020000 | LNF_UINT64 )
-#define LNF_FLD_RECEIVED		( 0x030000 | LNF_UINT64 )
-#define LNF_FLD_DOCTETS			( 0x040000 | LNF_UINT64 )
-#define LNF_FLD_DPKTS			( 0x050000 | LNF_UINT64 )
-#define LNF_FLD_OUT_BYTES		( 0x060000 | LNF_UINT64 )
-#define LNF_FLD_OUT_PKTS		( 0x070000 | LNF_UINT64 )
-#define LNF_FLD_AGGR_FLOWS		( 0x080000 | LNF_UINT64 )
-#define LNF_FLD_SRCPORT 		( 0x090000 | LNF_UINT16 )
-#define LNF_FLD_DSTPORT			( 0x0a0000 | LNF_UINT16 )
-#define LNF_FLD_TCP_FLAGS		( 0x0b0000 | LNF_UINT8 )
-#define LNF_FLD_SRCADDR 		( 0x0c0000 | LNF_ADDR )
-#define LNF_FLD_DSTADDR			( 0x0d0000 | LNF_ADDR )
-#define LNF_FLD_IP_NEXTHOP		( 0x0e0000 | LNF_ADDR )
-#define LNF_FLD_SRC_MASK		( 0x0f0000 | LNF_UINT8 )
-#define LNF_FLD_DST_MASK		( 0x100000 | LNF_UINT8 )
-#define LNF_FLD_TOS				( 0x110000 | LNF_UINT8 )
-#define LNF_FLD_DST_TOS			( 0x130000 | LNF_UINT8 )
-#define LNF_FLD_SRCAS			( 0x140000 | LNF_UINT32 )
-#define LNF_FLD_DSTAS			( 0x150000 | LNF_UINT32 )
-#define LNF_FLD_BGPNEXTADJACENTAS	( 0x160000 | LNF_UINT32 )
-#define LNF_FLD_BGPPREVADJACENTAS	( 0x170000 | LNF_UINT32 )
-#define LNF_FLD_BGP_NEXTHOP			( 0x180000 | LNF_ADDR )
-#define LNF_FLD_PROT	 		( 0x190000 | LNF_UINT8 )
-#define LNF_FLD_SRC_VLAN		( 0x200000 | LNF_UINT16 )
-#define LNF_FLD_DST_VLAN		( 0x210000 | LNF_UINT16 )
-#define LNF_FLD_IN_SRC_MAC		( 0x220000 | LNF_MAC )
-#define LNF_FLD_OUT_SRC_MAC		( 0x230000 | LNF_MAC )
-#define LNF_FLD_IN_DST_MAC		( 0x240000 | LNF_MAC )
-#define LNF_FLD_OUT_DST_MAC		( 0x250000 | LNF_MAC )
-#define LNF_FLD_MPLS_LABEL		( 0x260000 | LNF_MPLS )
-#define LNF_FLD_INPUT			( 0x270000 | LNF_UINT16 )
-#define LNF_FLD_OUTPUT			( 0x280000 | LNF_UINT16 )
-#define LNF_FLD_DIR				( 0x290000 | LNF_UINT8 )
-#define LNF_FLD_FWD_STATUS		( 0x300000 | LNF_UINT8 )
-#define LNF_FLD_IP_ROUTER		( 0x310000 | LNF_ADDR )
-#define LNF_FLD_ENGINE_TYPE		( 0x320000 | LNF_UINT8 )
-#define LNF_FLD_ENGINE_ID		( 0x3a0000 | LNF_UINT8 )
-#define LNF_FLD_EVENT_TIME		( 0x340000 | LNF_UINT64 )
-#define LNF_FLD_CONN_ID			( 0x350000 | LNF_UINT32 )
-#define LNF_FLD_ICMP_CODE		( 0x360000 | LNF_UINT8 )
-#define LNF_FLD_ICMP_TYPE		( 0x370000 | LNF_UINT8 )
-#define LNF_FLD_FW_XEVENT		( 0x380000 | LNF_UINT16 )
-#define LNF_FLD_XLATE_SRC_IP	( 0x390000 | LNF_ADDR )
-#define LNF_FLD_XLATE_DST_IP	( 0x400000 | LNF_ADDR )
-#define LNF_FLD_XLATE_SRC_PORT	( 0x410000 | LNF_UINT16 )
-#define LNF_FLD_XLATE_DST_PORT	( 0x420000 | LNF_UINT16 )
-#define LNF_FLD_INGRESS_ACL_ID	( 0x430000 | LNF_UINT32 )
-#define LNF_FLD_INGRESS_ACE_ID	( 0x440000 | LNF_UINT32 )
-#define LNF_FLD_INGRESS_XACE_ID	( 0x450000 | LNF_UINT32 )
-#define LNF_FLD_EGRESS_ACL_ID	( 0x460000 | LNF_UINT32 )
-#define LNF_FLD_EGRESS_ACE_ID	( 0x470000 | LNF_UINT32 )
-#define LNF_FLD_EGRESS_XACE_ID	( 0x480000 | LNF_UINT32 )
-#define LNF_FLD_USERNAME		( 0x490000 | LNF_STRING )
-#define LNF_FLD_INGRESS_VRFID	( 0x500000 | LNF_UINT32 )
-#define LNF_FLD_EVENT_FLAG		( 0x510000 | LNF_UINT8 )
-#define LNF_FLD_EGRESS_VRFID	( 0x520000 | LNF_UINT32 )
-#define LNF_FLD_BLOCK_START		( 0x530000 | LNF_UINT16 )
-#define LNF_FLD_BLOCK_END		( 0x540000 | LNF_UINT16 )
-#define LNF_FLD_BLOCK_STEP		( 0x550000 | LNF_UINT16 )
-#define LNF_FLD_BLOCK_SIZE		( 0x560000 | LNF_UINT16 )
-#define LNF_FLD_CLIENT_NW_DELAY_USEC	( 0x570000 | LNF_UINT64 )
-#define LNF_FLD_SERVER_NW_DELAY_USEC	( 0x580000 | LNF_UINT64 )
-#define LNF_FLD_APPL_LATENCY_USEC		( 0x590000 | LNF_UINT64 )
+#define LNF_FLD_FIRST			( 0x0100 | LNF_UINT64 )
+#define LNF_FLD_LAST			( 0x0200 | LNF_UINT64 )
+#define LNF_FLD_RECEIVED		( 0x0300 | LNF_UINT64 )
+#define LNF_FLD_DOCTETS			( 0x0400 | LNF_UINT64 )
+#define LNF_FLD_DPKTS			( 0x0500 | LNF_UINT64 )
+#define LNF_FLD_OUT_BYTES		( 0x0600 | LNF_UINT64 )
+#define LNF_FLD_OUT_PKTS		( 0x0700 | LNF_UINT64 )
+#define LNF_FLD_AGGR_FLOWS		( 0x0800 | LNF_UINT64 )
+#define LNF_FLD_SRCPORT 		( 0x0900 | LNF_UINT16 )
+#define LNF_FLD_DSTPORT			( 0x0a00 | LNF_UINT16 )
+#define LNF_FLD_TCP_FLAGS		( 0x0b00 | LNF_UINT8 )
+#define LNF_FLD_SRCADDR 		( 0x0c00 | LNF_ADDR )
+#define LNF_FLD_DSTADDR			( 0x0d00 | LNF_ADDR )
+#define LNF_FLD_IP_NEXTHOP		( 0x0e00 | LNF_ADDR )
+#define LNF_FLD_SRC_MASK		( 0x0f00 | LNF_UINT8 )
+#define LNF_FLD_DST_MASK		( 0x1000 | LNF_UINT8 )
+#define LNF_FLD_TOS				( 0x1100 | LNF_UINT8 )
+#define LNF_FLD_DST_TOS			( 0x1200 | LNF_UINT8 )
+#define LNF_FLD_SRCAS			( 0x1300 | LNF_UINT32 )
+#define LNF_FLD_DSTAS			( 0x1400 | LNF_UINT32 )
+#define LNF_FLD_BGPNEXTADJACENTAS	( 0x1500 | LNF_UINT32 )
+#define LNF_FLD_BGPPREVADJACENTAS	( 0x1600 | LNF_UINT32 )
+#define LNF_FLD_BGP_NEXTHOP			( 0x1700 | LNF_ADDR )
+#define LNF_FLD_PROT	 		( 0x1800 | LNF_UINT8 )
+#define LNF_FLD_SRC_VLAN		( 0x1900 | LNF_UINT16 )
+#define LNF_FLD_DST_VLAN		( 0x1a00 | LNF_UINT16 )
+#define LNF_FLD_IN_SRC_MAC		( 0x1b00 | LNF_MAC )
+#define LNF_FLD_OUT_SRC_MAC		( 0x1c00 | LNF_MAC )
+#define LNF_FLD_IN_DST_MAC		( 0x1d00 | LNF_MAC )
+#define LNF_FLD_OUT_DST_MAC		( 0x1e00 | LNF_MAC )
+#define LNF_FLD_MPLS_LABEL		( 0x1f00 | LNF_MPLS )
+#define LNF_FLD_INPUT			( 0x2000 | LNF_UINT16 )
+#define LNF_FLD_OUTPUT			( 0x2100 | LNF_UINT16 )
+#define LNF_FLD_DIR				( 0x2200 | LNF_UINT8 )
+#define LNF_FLD_FWD_STATUS		( 0x2300 | LNF_UINT8 )
+#define LNF_FLD_IP_ROUTER		( 0x2400 | LNF_ADDR )
+#define LNF_FLD_ENGINE_TYPE		( 0x2500 | LNF_UINT8 )
+#define LNF_FLD_ENGINE_ID		( 0x2600 | LNF_UINT8 )
+#define LNF_FLD_EVENT_TIME		( 0x2700 | LNF_UINT64 )
+#define LNF_FLD_CONN_ID			( 0x2800 | LNF_UINT32 )
+#define LNF_FLD_ICMP_CODE		( 0x2900 | LNF_UINT8 )
+#define LNF_FLD_ICMP_TYPE		( 0x2a00 | LNF_UINT8 )
+#define LNF_FLD_FW_XEVENT		( 0x2b00 | LNF_UINT16 )
+#define LNF_FLD_XLATE_SRC_IP	( 0x2c00 | LNF_ADDR )
+#define LNF_FLD_XLATE_DST_IP	( 0x2d00 | LNF_ADDR )
+#define LNF_FLD_XLATE_SRC_PORT	( 0x2e00 | LNF_UINT16 )
+#define LNF_FLD_XLATE_DST_PORT	( 0x2f00 | LNF_UINT16 )
+#define LNF_FLD_INGRESS_ACL_ID	( 0x3000 | LNF_UINT32 )
+#define LNF_FLD_INGRESS_ACE_ID	( 0x3100 | LNF_UINT32 )
+#define LNF_FLD_INGRESS_XACE_ID	( 0x3200 | LNF_UINT32 )
+#define LNF_FLD_EGRESS_ACL_ID	( 0x3300 | LNF_UINT32 )
+#define LNF_FLD_EGRESS_ACE_ID	( 0x3400 | LNF_UINT32 )
+#define LNF_FLD_EGRESS_XACE_ID	( 0x3500 | LNF_UINT32 )
+#define LNF_FLD_USERNAME		( 0x3600 | LNF_STRING )
+#define LNF_FLD_INGRESS_VRFID	( 0x3700 | LNF_UINT32 )
+#define LNF_FLD_EVENT_FLAG		( 0x3800 | LNF_UINT8 )
+#define LNF_FLD_EGRESS_VRFID	( 0x3900 | LNF_UINT32 )
+#define LNF_FLD_BLOCK_START		( 0x3a00 | LNF_UINT16 )
+#define LNF_FLD_BLOCK_END		( 0x3b00 | LNF_UINT16 )
+#define LNF_FLD_BLOCK_STEP		( 0x3c00 | LNF_UINT16 )
+#define LNF_FLD_BLOCK_SIZE		( 0x3d00 | LNF_UINT16 )
+#define LNF_FLD_CLIENT_NW_DELAY_USEC	( 0x3e00 | LNF_UINT64 )
+#define LNF_FLD_SERVER_NW_DELAY_USEC	( 0x3f00 | LNF_UINT64 )
+#define LNF_FLD_APPL_LATENCY_USEC		( 0x4000 | LNF_UINT64 )
 
-#define LNF_FLD_BREC1			( 0x910000 | LNF_BASIC_RECORD1 ) 	/* special field for lnf_brec1_t */
+#define LNF_FLD_BREC1			( 0x41000 | LNF_BASIC_RECORD1 ) 	/* special field for lnf_brec1_t */
 
 /* text description of fields */
 typedef struct lnf_field_s {
 	int index;			/* numerical index of field */
+	int default_aggr;	/* default aggregation function */
+	int default_sort;	/* default sort order */
 	char *name;			/* field name */
 	char *fld_descr;	/* short description */
 } lnf_field_t;
@@ -209,10 +215,10 @@ typedef void lnf_mem_t;
 #define LNF_WEAKERR	0x8		/* return weak erros $(unknow block, record) */
 
 /* other functions */
-void lnf_error(char *buf, int buflen);
+void lnf_error(const char *buf, int buflen);
 
 /* file operations */
-int lnf_open(lnf_file_t **lnf_filep, char *filename, unsigned int flags, char *ident);
+int lnf_open(lnf_file_t **lnf_filep, const char *filename, unsigned int flags, const char *ident);
 void lnf_info(lnf_file_t *lnf_file, lnf_info_t *lnf_info);
 int lnf_read(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec);
 int lnf_write(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec);
@@ -233,6 +239,7 @@ int	lnf_filter_init(lnf_filter_t **filterp, char *expr);
 int	lnf_filter_match(lnf_filter_t *filter, lnf_rec_t *rec);
 void lnf_filter_free(lnf_filter_t *filter);
 
+#define LNF_MAX_THREADS 128		/* maximum threads */
 
 /* memory heap operations */
 int lnf_mem_init(lnf_mem_t **lnf_mem);
@@ -250,8 +257,10 @@ int lnf_mem_init(lnf_mem_t **lnf_mem);
 #define LNF_SORT_DESC	0x0020	/* sort by item descending */
 #define LNF_SORT_FLAGS	0x00F0
 
-int lnf_mem_addf(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numbits6);
+int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numbits6);
 int lnf_mem_write(lnf_mem_t *lnf_mem, lnf_rec_t *rec);
+int lnf_mem_merge_threads(lnf_mem_t *lnf_mem);
+int lnf_mem_read(lnf_mem_t *lnf_mem, lnf_rec_t *rec);
 void lnf_mem_free(lnf_mem_t *lnf_mem);
 
 
