@@ -1,6 +1,6 @@
 #!/bin/sh 
 
-LIBNF_VERSION="1.03"
+LIBNF_VERSION="1.04"
 
 NFDUMP="nfdump-1.6.12"
 NFDUMP_MD5="e55a9130c93cfb9ed24b01bccd691bcb"
@@ -65,6 +65,8 @@ echo "Removing AC_OUTPUT and text from configure.ac"
 sed -i -e 's/AC_OUTPUT.*//g' configure.ac 	#makefiles
 sed -i -e 's/echo ".*//g' configure.ac 		#comments
 
+echo "Adding pthread.h checj into configure.ac" 
+perl -pi -w -e 's/AC_HEADER_STDC/AC_HEADER_STDC\nAC_CHECK_HEADERS(pthread.h)\nLIBS="\$LIBS -lpthread"/g;' configure.ac 
 
 echo "Adding extra configuration into configure.ac"
 cat >> configure.ac << EOT 
