@@ -13,7 +13,9 @@
 #include <rbtree.h>
 #include <nftree.h>
 #include <nfx.h>
+#include <pthread.h>
 #include "bit_array.h"
+#include "hash_table.h"
 
 
 /* list of maps used in file taht we create */
@@ -86,15 +88,51 @@ typedef struct lnf_fieldlist_s {
 	void *next;
 } lnf_fieldlist_t;
 
+#define LNF_MAX_THREADS 128			/* maximum threads */
 
 /* general memheap structure */
 typedef struct lnf_mem_s {
 	lnf_fieldlist_t *key_list;		/* list of fields with key values */
-	int	key_size;
+	int	key_len;
 	lnf_fieldlist_t *val_list;		/* list of fields with values (counters etc.) */
-	int	val_size;
-	lnf_fieldlist_t *sort_list;		/* list of fields to sort */
-	int	sort_size;
+	int	val_len;
+//	lnf_fieldlist_t *sort_list;		/* list of fields to sort */
+//	int	sort_len;
+	int sort_field;					/* field identification for sorting */
+	int sort_offset;				/* offset in the record field */
+	int sort_flags;					/* search sort field in ket or aggregated value */
+#define LNF_SORT_FLD_NONE 0x0
+#define LNF_SORT_FLD_IN_KEY 0x1
+#define LNF_SORT_FLD_IN_VAL 0x2
+	pthread_key_t thread_id_key;	/* key for thread specific id */
+	pthread_mutex_t thread_mutex;
+	int thread_status[LNF_MAX_THREADS];		/* status of the current thread */
+#define LNF_TH_WRITE 0x1			/* writing record - set after first write */
+#define LNF_TH_MERGE 0x2			/* writing done - ready for merge */
+#define LNF_TH_MERGING 0x3			/* merging thread */
+#define LNF_TH_MERGED 0x4			/* merged thread */
+#define LNF_TH_CLEARED 0x4			/* merge process done - hash table destroyed and ID cleared */
+	int numthreads;					/* participating number of threads */
+	hash_table_t hash_table[LNF_MAX_THREADS];	/* thread specific instance */
+//	char * hash_ptr;				/* row pointer for reading */
+	unsigned long read_index;		/* index for nex read */
+	int rearranged;					/* is the final hash table rearranged */
+	int sorted;						/* is the table sorted ? */
 } lnf_mem_t;
 
 
+#define LNF_MAX_KEY_LEN 512			/* maximum key length for hash table */
+#define LNF_MAX_VAL_LEN 256			/* maximum aggregated values length for hash table */
+
+int lnf_mem_thread_init(lnf_mem_t *lnf_mem);
+void lnf_filedlist_free(lnf_fieldlist_t *list);
+void lnf_clear_bits(char *buf, int buflen, int from);
+int lnf_mem_fill_buf(lnf_fieldlist_t *fld, lnf_rec_t *rec, char *buf);
+void lnf_mem_fill_rec(lnf_fieldlist_t *fld, char *buf, lnf_rec_t *rec);
+
+
+int lnf_filedlist_add(lnf_fieldlist_t **list, lnf_fieldlist_t *snode, int *sizep, int maxsize, int *offset);
+void lnf_mem_aggr_callback(char *key, char *hval, char *uval, void *lnf_mem);
+int lnf_mem_sort_callback(char *key1, char *val1, char *key2, char *val2, void *p);
+int lnf_mem_done(lnf_mem_t *lnf_mem);
+
